# CheapBombTimer release 6 by David Bowland
# ./addons/eventscripts/cheapbombtimer/cheapbombtimer.py

# >>> To configure this addon please see cheapbombtimer.cfg <<<

"""
Displays bomb time at specified intervals and emits a countdown voice from the bomb. No client downloads and compatible with slow motion! Requires only ES 2.0+
"""


import es
import langlib
import os.path
import playerlib
import random


info = es.AddonInfo()
info.name = 'CheapBombTimer'
info.version = '6'
info.url = 'http://addons.eventscripts.com/addons/view/cheapbombtimer'
info.basename = 'cheapbombtimer'
info.author = 'SuperDave'


int_bomb_tick = -1
int_bomb_tick_total = 0
int_bomb_time = 0
str_current_sound = ''
int_announce_interval = 0
int_announce_interval_count = 0
dict_intervals = {}
list_p2_info = []

dict_voice_sounds = {'0':{}, '1':{'30':'npc/metropolice/vo/off1.wav', '20':'npc/metropolice/vo/on2.wav', '15':'npc/metropolice/vo/off2.wav', '10':'npc/metropolice/vo/ten.wav', '9':'npc/metropolice/vo/nine.wav', '8':'npc/metropolice/vo/eight.wav', '7':'npc/metropolice/vo/seven.wav', '6':'npc/metropolice/vo/six.wav', '5':'npc/metropolice/vo/five.wav', '4':'npc/metropolice/vo/four.wav', '3':'npc/metropolice/vo/three.wav', '2':'npc/metropolice/vo/two.wav', '1':'npc/metropolice/vo/one.wav'}, '2':{'41':'npc/overwatch/cityvoice/f_anticivilevidence_3_spkr.wav', '30':'npc/overwatch/cityvoice/fcitadel_30sectosingularity.wav', '15':'npc/overwatch/cityvoice/fcitadel_15sectosingularity.wav', '10':'npc/overwatch/cityvoice/fcitadel_10sectosingularity.wav', '5':'npc/overwatch/radiovoice/five.wav', '4':'npc/overwatch/radiovoice/four.wav', '3':'npc/overwatch/radiovoice/three.wav', '2':'npc/overwatch/radiovoice/two.wav', '1':'npc/overwatch/radiovoice/one.wav'}, '3':{'40':'npc/combine_soldier/vo/fourty.wav', '35':'npc/combine_soldier/vo/thirty.wav', '35p2':'npc/combine_soldier/vo/five.wav', '35p2delay':0.3, '30':'npc/combine_soldier/vo/thirty.wav', '25':'npc/combine_soldier/vo/twenty.wav', '25p2':'npc/combine_soldier/vo/five.wav', '25p2delay':0.35, '20':'npc/combine_soldier/vo/twenty.wav', '15':'npc/combine_soldier/vo/fifteen.wav', '10':'npc/combine_soldier/vo/ten.wav', '9':'npc/combine_soldier/vo/niner.wav', '8':'npc/combine_soldier/vo/eight.wav', '7':'npc/combine_soldier/vo/seven.wav', '6':'npc/combine_soldier/vo/six.wav', '5':'npc/combine_soldier/vo/five.wav', '4':'npc/combine_soldier/vo/four.wav', '3':'npc/combine_soldier/vo/three.wav', '2':'npc/combine_soldier/vo/two.wav', '1':'npc/combine_soldier/vo/one.wav'}, '4':{'40':'HL1/fvox/fourty.wav', '35':'HL1/fvox/thirty.wav', '35p2':'HL1/fvox/five.wav', '35p2delay':0.6, '30':'HL1/fvox/thirty.wav', '25':'HL1/fvox/twenty.wav', '25p2':'HL1/fvox/five.wav', '25p2delay':0.6, '20':'HL1/fvox/twenty.wav', '15':'HL1/fvox/fifteen.wav', '10':'HL1/fvox/ten.wav', '5':'HL1/fvox/five.wav', '4':'HL1/fvox/warning.wav', '3':'HL1/fvox/evacuate_area.wav', '1':'HL1/fvox/immediately.wav'}}
dict_options = {'cheapbombtimer_bps':[83, 'Bomb beeps per second (x 10)'], 'cheapbombtimer_voice':['1;2;3;4', 'Possible bomb voices separated with a semicolon\n0 = none, 1 = metropolice, 2 = overwatch, 3 = combine soldier, 4 = HL1 suit'], 'cheapbombtimer_voice_coveragepercent':[50, 'Percent of the map around the bomb the voice can be heard'], 'cheapbombtimer_announce_chat':[0, '0 = no change, 1 = announce bomb time in chat area'], 'cheapbombtimer_announce_center':[1, '0 = no change, 1 = announce bomb time in center text'], 'cheapbombtimer_announce_hud':[0, '0 = no change, 1 = announce bomb time in a HUD hint'], 'cheapbombtimer_interval_at_plant':[45, 'Number of seconds between text announcements of the bomb time'], 'cheapbombtimer_bps_skipcheck':[0, '0 = no change, 1 = CheapBombTimer will not check the BPS value if the bomb explodes (only applies to current round)']}
func_lang_text = None


def load():
   """
   Creates commands for settings
   Checks to make sure critical server variables are created by the config
   """
   global func_lang_text

   if not es.exists('command', 'cheapbombtimer_interval_add'):
      es.regcmd('cheapbombtimer_interval_add', 'cheapbombtimer/interval_cmd', 'cheapbombtimer_interval_add <bomb time> <interval>\nServer command for designating bomb time announce intervals')
   if not es.exists('command', 'cheapbombtimer_interval_remove'):
      es.regcmd('cheapbombtimer_interval_remove', 'cheapbombtimer/interval_cmd', 'cheapbombtimer_interval_remove <bomb time>\nServer command for removing bomb time announce intervals')

   for str_option in dict_options:
      es.ServerVar(str_option, dict_options[str_option][0], dict_options[str_option][1])
   if os.path.isfile(es.getAddonPath('cheapbombtimer') + '/cheapbombtimer.cfg'):
      es.server.cmd('es_xmexec ../addons/eventscripts/cheapbombtimer/cheapbombtimer.cfg')
   else:
      es.dbgmsg(0, 'CheapBombTimer: Unable to load cheapbombtimer.cfg! Please ensure it is in the ./cheapbombtimer/ directory.')

   str_lang_path = es.getAddonPath('cheapbombtimer') + '/cheapbombtimer_languages.ini'
   if os.path.isfile(str_lang_path):
      func_lang_text = langlib.Strings(str_lang_path)
   else:
      es.dbgmsg(0, 'CheapBombTimer: Unable to load cheapbombtimer_languages.ini! Please ensure it is in the ./cheapbombtimer/ directory.')
   if not func_lang_text:
      func_lang_text = missing_lang


def round_start(event_var):
   """
   Sets int_bomb_tick to the before-plant value and resets cheapbombtimer_bps_skipcheck to zero
   Cancels the delay for the second part of the bomb voice
   """
   global int_bomb_tick

   int_bomb_tick = -1
   es.ServerVar('cheapbombtimer_bps_skipcheck').set(0)


def bomb_planted(event_var):
   """
   Resets count variables for the new bomb
   Chooses a random sound index for int_current_sound_index
   """
   global int_bomb_tick
   global int_bomb_tick_total
   global int_bomb_time
   global str_current_sound
   global int_announce_interval
   global int_announce_interval_count

   int_bomb_tick = 0
   int_bomb_tick_total = 0
   int_bomb_time = int(es.ServerVar('mp_c4timer'))
   int_announce_interval = int(es.ServerVar('cheapbombtimer_interval_at_plant'))
   int_announce_interval_count = 0

   list_voice_choices = str(es.ServerVar('cheapbombtimer_voice')).split(';')
   if str_current_sound in list_voice_choices and len(list_voice_choices) > 1:
      list_voice_choices.remove(str_current_sound)
   str_current_sound = random.choice(list_voice_choices)


def bomb_beep(event_var):
   """
   Increments int_bomb_tick and int_bomb_tick_total
   Displays bomb time if necessary
   Emits voice from bomb if necessary
   """
   global int_bomb_tick
   global int_bomb_tick_total
   global int_bomb_time
   global int_announce_interval
   global int_announce_interval_count
   global list_p2_info

   if int_bomb_tick >= 0:
      int_bomb_tick += 10
      int_bomb_tick_total += 10
      if int_bomb_tick > es.ServerVar('cheapbombtimer_bps'):
         int_bomb_tick -= int(es.ServerVar('cheapbombtimer_bps'))
         int_bomb_time -= 1

         int_announce_interval_count += 1
         if dict_intervals.has_key(int_bomb_time):
            int_announce_interval_count = 0
            int_announce_interval = dict_intervals[int_bomb_time]
         if not (int_announce_interval_count % int_announce_interval):
            int_announce_chat = int(es.ServerVar('cheapbombtimer_announce_chat'))
            int_announce_center = int(es.ServerVar('cheapbombtimer_announce_center'))
            int_announce_hud = int(es.ServerVar('cheapbombtimer_announce_hud'))
            if int_announce_chat or int_announce_center or int_announce_hud:
               dict_time_token = {'time':str(int_bomb_time)}
               for int_userid in es.getUseridList():
                  str_text = func_lang_text('bomb time', dict_time_token, playerlib.getPlayer(int_userid).get('lang'))
                  if int_announce_chat:
                     es.tell(int_userid, str_text)
                  if int_announce_center:
                     es.centertell(int_userid, remove_tags(str_text))
                  if int_announce_hud:
                     es.usermsg('create', 'cheapbombtimer_hud', 'HintText')
                     es.usermsg('write', 'short', 'cheapbombtimer_hud', -1)
                     es.usermsg('write', 'string', 'cheapbombtimer_hud', remove_tags(str_text))
                     es.usermsg('send', 'cheapbombtimer_hud', int_userid)
                     es.usermsg('delete', 'cheapbombtimer_hud')

         if dict_voice_sounds[str_current_sound].has_key(str(int_bomb_time)):
            float_attenuation = (100 - float(es.ServerVar('cheapbombtimer_voice_coveragepercent'))) / 100
            es.emitsound('entity', event_var['entindex'], dict_voice_sounds[str_current_sound][str(int_bomb_time)], 1.0, float_attenuation)
            if dict_voice_sounds[str_current_sound].has_key(str(int_bomb_time) + 'p2'):
               list_p2_info = [int(event_var['entindex']), dict_voice_sounds[str_current_sound][str(int_bomb_time) + 'p2'], float_attenuation]
               es.delayed(dict_voice_sounds[str_current_sound][str(int_bomb_time) + 'p2delay'], 'es_xdoblock cheapbombtimer/delayed_sound')


def bomb_exploded(event_var):
   """Checks cheapbombtimer_bps against the number of ticks counted before explosion"""
   if int_bomb_tick > 0 and not int(es.ServerVar('cheapbombtimer_bps_skipcheck')):
      int_new_bps = round(float(int_bomb_tick_total + 10) / float(es.ServerVar('mp_c4timer')))
      if int_new_bps <> es.ServerVar('cheapbombtimer_bps'):
         es.centermsg('cheapbombtimer_bps incorrectly configured!')
         es.msg('cheapbombtimer_bps incorrectly configured!')
         es.msg('You should change your cheapbombtimer_bps value to %s' % int_new_bps)
         es.msg('This setting can found in your cheapbombtimer.cfg')
         es.ServerVar('cheapbombtimer_bps').set(int_new_bps)


def unload():
   for str_option in dict_options:
      es.ServerVar(str_option).set(0)


def delayed_sound():
   """ """
   global list_p2_info

   int_index = list_p2_info[0]
   if es.createentitylist(int_index).has_key(int_index):
      es.emitsound('entity', int_index, list_p2_info[1], 1.0, list_p2_info[2])
   list_p2_info[0] = -1


def interval_cmd():
   """
   cheapbombtimer_interval_add <bomb time> <interval>
   cheapbombtimer_interval_remove <bomb time>
   Adds or removes bomb time announce intervals
   """
   global dict_intervals

   str_command = es.getargv(0).lower()
   if es.getargc() == {'cheapbombtimer_interval_add':3, 'cheapbombtimer_interval_remove':2}[str_command]:
      int_time_arg = int(es.getargv(1))
      if str_command == 'cheapbombtimer_interval_add':
         dict_intervals[int_time_arg] = int(es.getargv(2))

      elif dict_intervals.has_key(int_time_arg):
         del dict_intervals[int_time_arg]

   else:
      es.dbgmsg(0, 'Syntax: ' + {'cheapbombtimer_interval_add':'cheapbombtimer_interval_add <bomb time> <interval>', 'cheapbombtimer_interval_remove':'cheapbombtimer_interval_remove <bomb time>'}[str_command])


def remove_tags(str_text):
   """Removes #lightgreen, #green and #default tags from the supplied string"""
   return str_text.replace('#lightgreen', '').replace('#green', '').replace('#default', '')


def missing_lang(str_heading, dict_tokens, str_language):
   """Displays an error message because the languages .ini could not be found"""
   return 'CheapBombTimer: cheapbombtimer_languages.ini not found'